package org.jvisu.panel;

import java.awt.*;
import java.awt.image.*;
import java.beans.*;
import java.net.*;
import javax.swing.*;
import javax.swing.border.*;
import org.jvisu.comm.var.*;

/**
 *  <p>package:    org.jvisu.panel</p>
 *  <p>file:       JVSubPanel.java</p>
 * $Id: JVSubPanel.java 318 2007-08-21 11:23:19Z mol $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 * JVSubPanel can hold JVisu components and can control the visiblity
 * by a boolean variable. Also the editenable of all subcomponents can be
 * controlled by a variable.
 * They layout of the container can be controlled by the user.
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.3
 */
public class JVSubPanel extends JVPanel {
    
    /** holds the title of this panel */
    private String                  title;
    /** frame color */
    private Color                   frameColor;
    /** Selects, if visibility is controlled by a variable*/
    private boolean                 visibilityByVariable;
    /** Invers boolean of visiblity */
    private boolean                 visInvers;
    /** Selects, if editEnabled is controlled by a variable*/
    private boolean                 editEnabledByVariable;
    /** Invers boolean of editbility */
    private boolean                 editInvers;

    /** Layout selection              */
    private int                     layoutType = LAYOUT_FIXED;
    /** Rows for grid layout        */
    private int                     rows = 1;
    /** Columns for grid layout        */
    private int                     cols = 1;

    /** The layout type constant for flow layout */
    public final static int         LAYOUT_FLOW = 0;
    /** The layout type constant for fixed layout */
    public final static int         LAYOUT_FIXED = 1;
    /** The layout type constant for box layout x */
    public final static int         LAYOUT_BOXX = 2;
    /** The layout type constant for box layout y */
    public final static int         LAYOUT_BOXY = 3;
    /** The layout type constant for grid layout */
    public final static int         LAYOUT_GRID = 4;
    
    /** Creates a new instance of JVSubPanel.
     */
    public JVSubPanel() {
        addCommVar (new JVVarCfg(null, updaterate, JVVar.T_BOOL, 0, 0));
        addCommVar (new JVVarCfg(null, updaterate, JVVar.T_BOOL, 0, 1));
        modifyLayout();
        setPreferredSize(new Dimension(100,100));
    }
    
    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        
        super.propertyChange(evt);
        String key = evt.getPropertyName();
        if (key.equals("font")) {
            if (getBorder() instanceof TitledBorder)
                ((TitledBorder)getBorder()).setTitleFont(getFont());
        }
    }
    /** Set the layout for this component
     */
    private void modifyLayout() {
        switch (layoutType) {
            case LAYOUT_FLOW:
                setLayout(new FlowLayout());
                break;

            case LAYOUT_FIXED:
                setLayout(new SpringLayout());
                break;

            case LAYOUT_BOXX:
                setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
                break;

            case LAYOUT_BOXY:
                setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
                break;

            case LAYOUT_GRID:
                setLayout(new GridLayout(rows, cols));
                break;
        } 
        this.revalidate();
    }
    
    /** get the title for this component
     * @return title
     */
    public String getTitle() {
        return title;
    }
    
    /** set the title for this component
     * @param tit the new title
     */
    public void setTitle(String tit) {
        if (tit != null && tit.equals(title)) 
            return;
        if (title != null && title.equals(tit)) 
            return;

        title = tit;
        modifyBorder();
    }

    /** Setter for property frameColor.
     *  Change the border type depending on the given value
     *  @param frameColor New value of property frameColor.
     */
    public void setFrameColor(Color frameColor) {
        
        if(this.frameColor != frameColor) {
            this.frameColor = frameColor;
            modifyBorder();
        }
    }

    /** Modify the border
     */
    private void modifyBorder() {
        if (title != null) {
            if (frameColor == null)
                setBorder(new TitledBorder(title));
            else {
                setBorder(new TitledBorder(title));
                ((TitledBorder)getBorder()).setBorder(new LineBorder(frameColor));
            }
            ((TitledBorder)getBorder()).setTitleFont(getFont());
        } else {
            if (frameColor == null)
                setBorder(null);
            else
                setBorder(new LineBorder(frameColor));
        }
    }

    /** Not used here */
    public void Shutdown() {
    }
    
    /** Not used here */
    public void StartUp() {
    }
    
    /** This is called, if one of the registered variables
     *  has been changed.
     *  @param var the registered communication variable
     */
    public void VarChanged(org.jvisu.comm.var.JVVar var) {
        
        if (getCommVarCfg(0).getSymbol() != null &&
            getCommVarCfg(0).getSymbol().equals(var.getVarCfg().getSymbol())) {
            
            if (visibilityByVariable ^ visInvers)
                this.setVisible(((JVVarBool) var).getVarValue());
            
        } else if (getCommVarCfg(1).getSymbol() != null &&
            getCommVarCfg(1).getSymbol().equals(var.getVarCfg().getSymbol())) {
            
            if (editEnabledByVariable ^ editInvers)
                super.setEditEnabled(((JVVarBool) var).getVarValue());
        }
    }

   /** Control, if visibility is controlled by a variable
     *  @param val true, if visibility is controlled by a variable
    */
    public void setVisibilityByVariable (boolean val) {
        visibilityByVariable = val;
    }

    /** Return setting, if visibility is controlled by a variable
     *  @return true, if visibility is controlled by a variable
    */
    public boolean getVisibilityByVariable () {
        return visibilityByVariable;
    }

    /** Control, if Editenabled is controlled by a variable
     *  @param val true, if Editenabled is controlled by a variable
    */
    public void setEditEnabledByVariable (boolean val) {
        editEnabledByVariable = val;
    }

    /** Return setting, if editEnabled is controlled by a variable
     *  @return true, if editEnabled is controlled by a variable
    */
    public boolean getEditEnabledByVariable () {
        return editEnabledByVariable;
    }

    /** Set variable value, send it to server, if possible 
     *  @param val new value for visibility
    */
    public void setVisibility (boolean val) {
        setCommVarValue (0, new Boolean (val));
    }
    
    /** Get variable value 
     *  @return the variable content
    */
    public boolean getVisibility () {
       	return this.isVisible();
    }
    
    /** Get the symbol of the communication variable, which 
     *  controls the visibility of this component
     * @return the variable symbol of this component
     */
    public String getVisibilityVar() {
        if (visInvers)
            return "!" + getCommVarSymbol(0);    
        else
            return getCommVarSymbol(0);    
    }
    
    /** Initialize commvariable, which controls the visibility of this component
     * @param symbol the variable symbol or null
     */
    public void setVisibilityVar(String symbol){
        if (symbol != null && symbol.charAt(0) == '!') {
            visInvers = true;
            setCommVarSymbol (0, symbol.substring(1));
        } else {
            visInvers = false;
            setCommVarSymbol (0, symbol);
        }
    }

    /** Get the symbol of the communication variable, which 
     *  controls the editEnabled property
     * @return the variable symbol of this component
     */
    public String getEditEnabledVar() {
        if (editInvers)
            return "!" + getCommVarSymbol(1);    
        else
            return getCommVarSymbol(1);    
    }
    
    /** Initialize commvariable, which controls the editEnabled property
     * @param symbol the variable symbol or null
     */
    public void setEditEnabledVar(String symbol){
        if (symbol != null && symbol.charAt(0) == '!') {
            editInvers = true;
            setCommVarSymbol (1, symbol.substring(1));
        } else {
            editInvers = false;
            setCommVarSymbol (1, symbol);
        }
    }

    /** Get the layout type of this container
     * @return the layout type
     */
    public int getLayoutType() {
        return layoutType;
    }
    
    /** Set the layout type of this container
     * @param type the layout type
     */
    public void setLayoutType(int type){
        int old = layoutType;
        layoutType = type;
        modifyLayout();
        firePropertyChange("layoutType", old, type);
    }
    
    /** Get the grid rows for gridlayout
     * @return the grid rows
     */
    public int getGridRows() {
        return rows;
    }

    /** Set the grid rows for gridlayout
     * @param val the grid rows
     */
    public void setGridRows(int val) {
        if (rows != val) { 
            rows = val;
            if (layoutType == this.LAYOUT_GRID) {
                ((GridLayout)getLayout()).setRows(val);
                this.revalidate();
            }
        }
    }

    /** Get the grid columns for gridlayout
     * @return the grid columns
     */
    public int getGridColumns() {
        return cols;
    }

    /** Set the grid columns for gridlayout
     * @param val the grid columns
     */
    public void setGridColumns(int val) {
        if (cols != val) {
            cols = val;
            if (layoutType == this.LAYOUT_GRID) {
                ((GridLayout)getLayout()).setColumns(val);
                this.revalidate();
            }
        }
    }

    /** Setter for property editEnable. As this component can control 
     *  editEnabled with a variable, the method in the superclass is
     *  overridden. If the control by variable is off, the value can
     *  be accepted, else it is thrown away.
     * @param editEnable New value of property editEnable.
     */
    public void setEditEnabled(boolean editEnable) {
        if (!editEnabledByVariable)
            super.setEditEnabled(editEnable);
    }
    
    /** Main method for standalone testing */
     public static void main(String args[]) {
        final JFrame frame = new JFrame();
        frame.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                System.exit(0);
            }
        });
        frame.getContentPane().setLayout(new BoxLayout(frame.getContentPane(), BoxLayout.Y_AXIS));
        
        final JVSubPanel pan = new JVSubPanel();
        pan.setLayoutType(JVSubPanel.LAYOUT_FIXED);
        pan.setServer("localhost");
        pan.setCommtype("org.jvisu.comm.JVSockComm");
     
        final org.jvisu.base.JVBool jb = new org.jvisu.base.JVBool();
        jb.setCommVar("outbit0");
        jb.setPort(2201);
        jb.setReadonly(false);
        pan.add(jb);
        
        JButton b = new JButton("test");
        b.addActionListener(new java.awt.event.ActionListener () {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                pan.setEditEnabled(!jb.isEditEnabled());        
                jb.setDesignpos(new Point(40, 40));
            }
        });
        frame.getContentPane().add(b);
        
        frame.getContentPane().add(pan);
        frame.pack();
        frame.setVisible(true);
    }    
}
