package org.jvisu.panel.configurator;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.io.*;
import java.net.*;
import org.jvisu.base.*;
import org.jvisu.util.*;
import org.jvisu.panel.*;
import org.jvisu.panel.configurator.events.*;

/**
 *  <p>package:    JVisu</p>
 *  <p>file:       JVPanelConfigurator.java</p>
 * $Id: JVPanelConfigurator.java 319 2007-08-21 12:11:50Z mol $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 * JVPanelConfigurator is the designer application for JVisu
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.3
 */
public class JVPanelConfigurator extends javax.swing.JFrame implements 
            TreeSelectionListener, ActionListener, JVEditEventListener {

    /** Maximum undo size */
    private int                     maxundosize = 100;
                
    /** The configuration file */
    private String                  visuFile;
    /** Filechooser */
    private JFileChooser            fileChooser;
    /** Icon for the top node */
    private ImageIcon               topnodeIcon;
    /** The menubar */
    private JVPConfMenu             menubar;
    /** The edit popup */
    private JPopupMenu              editPopup;
    /** Stringreader for cut and paste */
    private StringReader            cutpaste;
    /** Undo array */
    private ArrayList               undo = new ArrayList();
    /** Redo array */
    private ArrayList               redo = new ArrayList();
    /** The undo active status */
    private boolean                 undoactive;
    /** The redo active status */
    private boolean                 redoactive;
    
    /** The design elements */
    private JVPalettePanel          PalettePanel;
    private JVPanelEditor           PanelEditor;
    private JVPanelPropertyEditor   PanelPropertyEditor;
    private JVPanelTree             PanelTree;
    
    /** Creates new form JVPanelConfigurator */
    public JVPanelConfigurator() {
        /*
        try {
            UIManager.setLookAndFeel(new com.incors.plaf.kunststoff.KunststoffLookAndFeel());
        } catch (Exception e) { 
            // Not there, let it be metal 
        }
        */
        initComponents();
        fileChooser = new JFileChooser();
        fileChooser.setCurrentDirectory(new File(("/" + System.getProperty("user.dir") + "/")));

        buildPalette(("file:///" + System.getProperty("user.dir") + "/panelcfg.xml"));

        PanelTree.getTree().setCellRenderer(new JVPanelTreeRenderer());
        /* The toplevel object in the tree view is JVPanelEdit.
         */
        topnodeIcon = new ImageIcon(getClass().getResource("/org/jvisu/panel/configurator/icons/jvisu.png"));
        PanelTree.setTopObject(new JVPanelTreeComponent(PanelEditor.getEditPanel(), topnodeIcon, "JVBasePanel"));

        PanelTree.addTreeSelectionListener(this);

        PanelEditor.addEditEventListener(this);
        PalettePanel.addPropertyChangeListener("selectedElement", PanelEditor);
        
        PanelPropertyEditor.addEditEventListener(this);

        menubar = new JVPConfMenu();
        try {
            URL url = new URL("file:///" + System.getProperty("user.dir") + "/configmenus.xml");
            menubar.parseConfigFile(url, this);
        } catch (Exception e) {
            try {
                URL url = getClass().getResource(
                    "/org/jvisu/panel/configurator/defconfigs/configmenus.xml");
                menubar.parseConfigFile(url, this);
            } catch (Exception e1) {
                System.err.println("Menu configuration not found");
                System.exit(-1);
            }
        }
        setJMenuBar(menubar);
        
        /** Get the toolbar */
        JToolBar toolbar = menubar.getToolbar();
        getContentPane().add(toolbar,java.awt.BorderLayout.NORTH);
        
        /** Get the popupmenu */
        editPopup = menubar.getPopup();
        PanelEditor.setPopupMenu(editPopup);
        PanelTree.setPopupMenu(editPopup);

        pack();
        
        Dimension size = getSize();
        Dimension screen = getToolkit().getScreenSize();
        setLocation((screen.width - size.width) / 2, (screen.height - size.height) / 2);
    }
    
    /** Return the version string 
     */
    public static String getVersion () {
        return "JVisu Panel Configuration Tool V1.0";
    }
    
    /** Static command list 
     *  Please do not change the order, as the switch/case relies on this list
     *  Just append stuff. Maybe you have a better idea :)
     */
    private String[] cmdList = {
        "FileOpen", 
        "FileSave",
        "FileSaveAs",
        "FileExit",
        "EditCut",
        "EditCopy",
        "EditPaste",
        "EditDelete",
        "OptionsShowAll",
        "OptionsLanguage0",
        "OptionsLanguage1",
        "FileNew",
        "OptionsPreview",
        "OptionsZoomIn",
        "OptionsZoomOut",
        "OptionsZoomFit",
        "EditUndo",
        "EditRedo",
        "OptionsZoomReal",
        "HelpAbout",
    };
    
    /** The action performer for this form 
     *  @param evt the action event to perform
     */
    public void actionPerformed(java.awt.event.ActionEvent evt) {
        
        String cmd = evt.getActionCommand();
        int i = 0;
        for (; i < cmdList.length; i++) {
            if (cmd.compareTo(cmdList[i]) == 0)
                break;
        }
        if (i == cmdList.length)
            return;

        switch (i) {
            // Open
            case 0:
                if (fileChooser.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
                    break;
                try {
                    visuFile = fileChooser.getSelectedFile().toURI().toURL().toString();
                    this.readvisuFile();
                } catch (Exception e) { }
                break;

            // Save. Fall through, if no filename given    
            case 1:
                if (visuFile != null) {
                    writevisuFile();
                    break;
                }

            // Save as. 
            case 2:
                if (fileChooser.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
                    break;
                try {
                    visuFile = fileChooser.getSelectedFile().toURI().toURL().toString();
                    this.writevisuFile();
                } catch (Exception e) { 
                    System.out.println (e);
                }
                break;

            // Exit
            case 3: {
                System.exit(0);
                break;
            }
            
            // Cut
            case 4: 
                processCut();
                break;
            
            // Copy
            case 5:
                cutpaste = null;
                if (PanelTree.getSelectedNode() != PanelTree.getTopNode()) {
                    DefaultMutableTreeNode node = PanelTree.getSelectedNode();
                    if (node != null)
                        cutpaste = nodesToXML(PanelTree.getSelectedNode());
                break;
            }
            
            // Paste
            case 6: {
                JVPanelTreeComponent treec = (JVPanelTreeComponent) PanelTree.getSelectedObject();
                if (treec != null && cutpaste != null) {
                    JComponent c = (JComponent)treec.getObject();
                    addUndoEvent(new JVEditEvent(this,JVEditEvent.BULK_START, null));
                    PanelEditor.parseStream(c, cutpaste);                
                    addUndoEvent(new JVEditEvent(this,JVEditEvent.BULK_END, null));
                }
                break;
            }
            
            // Delete
            case 7:
                processDelete();
                break;

            // Show all properties
            case 8:
                // PanelPropertyEditor.setShowAllProperties(optionShowAllMenuItem.isSelected());
                break;
            
            // Change language
            case 9:
                JVLanguageSelector.setLanguageSelector(0, this);
                break;
            
            // Change language
            case 10:
                JVLanguageSelector.setLanguageSelector(1, this);
                break;
                
            // New file    
            case 11:     
                visuFile = null;
                readvisuFile();
                break;

            // Preview
            case 12:
                PanelEditor.setPreviewMode(((AbstractButton)evt.getSource()).isSelected());
                break;
                
            // Zoom in
            case 13: {
                JVPanelEdit p = PanelEditor.getEditPanel();
                if (p.getZoomfactor() <= 2.25)
                    p.setZoomfactor(p.getZoomfactor() + 0.25);
                else
                    p.setZoomfactor(2.5);
                break;
            }
            // Zoom out
            case 14: {
                JVPanelEdit p = PanelEditor.getEditPanel();
                if (p.getZoomfactor() >= 0.5)
                    p.setZoomfactor(p.getZoomfactor() - 0.25);
                else
                    p.setZoomfactor(0.25);
                break;
            }
            // Zoom to fit window
            case 15: 
                PanelEditor.fitSize();
                break;
            // Undo
            case 16:
                processUndo();
                break;
            // Undo
            case 17:
                processRedo();
                break;
            // Zoom to real size
            case 18: 
                PanelEditor.getEditPanel().setZoomfactor(1);
                break;
            // Help about 
            case 19:
                new org.jvisu.panel.configurator.dialogs.JVAboutDlg(this).setVisible(true);
                break;
        } // End switch
        
        // Update the actions enable / disable
        Action a = menubar.getActionMap().get("EditPaste");
        if (a != null)
            a.setEnabled(cutpaste != null);
    }
    
    /** Process a delete action
     */
    private void processDelete() {
        if (PanelTree.getSelectedNode() == PanelTree.getTopNode())
            return;

        JVPanelTreeComponent treec = (JVPanelTreeComponent) PanelTree.getSelectedObject();
        if (treec == null)
            return;
        JComponent c = (JComponent)treec.getObject();
        addUndoEvent(new JVEditEvent(this,JVEditEvent.COMP_REMOVED, (JComponent)c.getParent(), c));
        PanelEditor.removeElement(c); 
    }

    /** Process a cut action
     */
    private void processCut() {
        cutpaste = null;
        if (PanelTree.getSelectedNode() == PanelTree.getTopNode())
            return;
        DefaultMutableTreeNode node = PanelTree.getSelectedNode();
        if (node == null)
            return;
        JVPanelTreeComponent treec = (JVPanelTreeComponent) PanelTree.getSelectedObject();
        JComponent c = (JComponent)treec.getObject();
        addUndoEvent(new JVEditEvent(this,JVEditEvent.COMP_REMOVED, (JComponent)c.getParent(), c));
        cutpaste = nodesToXML(PanelTree.getSelectedNode());
        JComponent comp = (JComponent) ((JVPanelTreeComponent) node.getUserObject()).getObject();
        /* Remove the component and all subcomponents */    
        PanelEditor.removeElement(comp); 
    }

    /** Process a single undo event 
     */
    private void processUndo() {
        undoactive = true;
        processUndoRedo(undo, false);
        Action a = menubar.getActionMap().get("EditUndo");
        a.setEnabled(undo.size() > 0);
        undoactive = false;
    }

    /** Process a single redo event 
     */
    private void processRedo() {
        redoactive = true;
        processUndoRedo(redo, false);
        Action a = menubar.getActionMap().get("EditRedo");
        a.setEnabled(redo.size() > 0);    
        redoactive = false;
    }

    /** Process a single undo event 
     *  @param list the arraylist, which is used
     *  @param bulk true, if a bulk undo is in progress
     */
    private boolean processUndoRedo(ArrayList list, boolean bulk) {
        if (list.size() == 0)
            return false;
        /* Get the event */
        JVEditEvent e = (JVEditEvent) list.remove(0);
        switch (e.getType()) {
            
            case JVEditEvent.PROP_CHANGED: {
                JVEditEvent.PropertyChange pc =(JVEditEvent.PropertyChange)e.getObject();
                try {
                    JVBasePanel.setProperty(pc.getComponent(), pc.getName(), pc.getValue());
                } catch (Exception ex) { 
                    System.err.println(ex);
                }
                break;
            }
            
            case JVEditEvent.COMP_ADDED: {
                JVEditEvent.CompAdd ac =(JVEditEvent.CompAdd)e.getObject();
                try {
                    ac.getParent().remove(ac.getChild());
                } catch (Exception ex) { 
                    System.err.println(ex);
                }
                e.setType(JVEditEvent.COMP_REMOVED);
                addUndoEvent(e);
                break;
            }       
            
            case JVEditEvent.COMP_REMOVED: {
                JVEditEvent.CompAdd ac =(JVEditEvent.CompAdd)e.getObject();
                try {
                    ac.getParent().add(ac.getChild());
                    PanelEditor.getEditPanel().revalidate();
                    updateAddedNodes(ac.getChild());
                } catch (Exception ex) { 
                    System.err.println(ex);
                }
                break;
            }
            
            case JVEditEvent.BULK_END: {
                e.setType(JVEditEvent.BULK_START);
                addUndoEvent(e);
                while (processUndoRedo(list,true));
                break;
            }

            case JVEditEvent.BULK_START: {
                e.setType(JVEditEvent.BULK_END);
                addUndoEvent(e);
                bulk = false;
                break;
            }
        }
        return bulk;
    }

    /** Update node, which is added by a COMP_REMOVED undo/redo event
     *  @param comp component to scan for childs
     */
    private void updateAddedNodes(JComponent comp) {
        for (int i = 0; i < comp.getComponentCount(); i++) {
            try {
                JVBaseMembers child = (JVBaseMembers) comp.getComponent(i);
                addCompToTree((JComponent) child);
                if (child.isContainer()) {
                    ((JComponent)child).revalidate();
                    updateAddedNodes((JComponent) child);
                }
            } catch (ClassCastException e) { }
        }
    }
    
    /** Add an event to the undo list 
     * @param evt the edit event
     */
    private void addUndoEvent(JVEditEvent e) {
        if (!undoactive) {
            //System.out.println ("add undo event: " + e.getType());
            undo.add(0, e);
            /* if maximum size reached, remove last element */
            if (undo.size() > maxundosize)
                undo.remove(maxundosize);
            if (!redoactive)
                redo.clear();
        } else {
            //System.out.println ("add redo event " + e.getType());
            redo.add(0, e);
            /* if maximum size reached, remove last element */
            if (redo.size() > maxundosize)
                redo.remove(maxundosize);
        }
        Action a = menubar.getActionMap().get("EditUndo");
        a.setEnabled(undo.size() > 0);
        a = menubar.getActionMap().get("EditRedo");
        a.setEnabled(redo.size() > 0);    
    }
    
    /** Edit event is fired
     * @param evt the edit event
     */
    public void editEventFired(JVEditEvent e) {
        if (e.getSource() == PanelEditor)
            checkPanelEditorEvent(e);
        else if (e.getSource() == PanelPropertyEditor)
            addUndoEvent(e);
        else
            System.out.println (e.getSource());
    }
    
    /* Eventlistener for Panel editor events 
     *  @param e the editor event
     */
    private void checkPanelEditorEvent(JVEditEvent e) {

        switch (e.getType()) {
            case JVEditEvent.COMP_SELECTED: {
                JComponent c = (JComponent) e.getObject();
                PanelTree.setSelectedObject(c);
                // Fall through and check, if an element must be added
            }
            case JVEditEvent.COMP_CLICKED: {
                JComponent c = (JComponent) e.getObject();
                if (c == null)
                    break;
                JVPalettePanel.JVPaletteAction descr = PalettePanel.getSelection();
                if (descr == null)
                    break;
                StringReader in = this.compToXML(descr.getRefComponent());
                if (in != null)
                    PanelEditor.parseStream(c, in);
                break;
            }
            case JVEditEvent.COMP_ADDED: {
                JComponent c = (JComponent) e.getObject();
                addCompToTree(c);
                addUndoEvent(new JVEditEvent(this,JVEditEvent.COMP_ADDED, (JComponent) c.getParent(), c));
                break;
            }
            case JVEditEvent.COMP_REMOVED: {
                PanelTree.removeObject(e.getObject());
                break;
            }
        }
        PalettePanel.clearSelection();
    }    

    /** Add component to tree 
     * @param comp the component, which must be added to the tree view
     */
    private void addCompToTree(JComponent comp) {
        JVPalettePanel.JVPaletteAction a = PalettePanel.getActionForDesignModel(comp.getClass().getName());
        ImageIcon icon = null;
        if (a != null)
            icon = (ImageIcon) a.getValue(AbstractAction.SMALL_ICON);
        if (icon == null)
            icon = topnodeIcon;
        JVPalettePanel.JVPaletteAction action = PalettePanel.getActionForDesignModel(comp.getClass().getName());
        String name = action.getValue(AbstractAction.NAME).toString();
        JVPanelTreeComponent tc = new JVPanelTreeComponent(comp, icon, name + " ");
        PanelTree.addObject(comp.getParent(), tc, true);
        PanelTree.setSelectedObject(comp);
    }
    
    
    /** Build the item palette 
     *  @param filename file name to read configuration
     */
    private void buildPalette(String filename) {
        URL url;
        try {
            url = new URL (filename);
            PalettePanel.parsePaletteFile(url);
        } catch (Exception e) { 
            try {
                PalettePanel.parsePaletteFile(
                    getClass().getResource(
                    "/org/jvisu/panel/configurator/defconfigs/panelcfg.xml"));
            } catch (Exception e1) { 
            }
        }
    }

    /** Get component properties
     *  @param comp Component to look up property descriptors
     *  @return arraylist of property descriptors
     */
    private ArrayList getDescriptors (JComponent comp) {
        try {
            if (comp instanceof JVBaseMembers)
                return PalettePanel.getActionForDesignModel(
                    ((JVBaseMembers)comp).getDesignModel()).getProperties();
            else 
                return PalettePanel.getActionForDesignModel(
                    comp.getClass().getName()).getProperties();
        } catch (Exception e) {
            return null;
        }
    }
    
    /* Eventlistener for TreeSelection events 
     * @param e the tree selection event
     */
    public void valueChanged(TreeSelectionEvent e) {
        JVPanelTreeComponent treec = (JVPanelTreeComponent) PanelTree.getSelectedObject();
        if (treec != null) {
            JComponent c = (JComponent)treec.getObject();
            PanelEditor.setSelectedComponent (c);
            PanelPropertyEditor.setEditComponent(c, getDescriptors(c));
        }
    }
    
    /** Try to read the configuration file */
    private void readvisuFile() {
        PanelTree.removeTreeSelectionListener(this);
        PanelTree.removeAllObjects();
        PanelEditor.parseFile((visuFile));
        PanelTree.addTreeSelectionListener(this);
    }

    /** Produce an XML representation of a part of the nodetree.
     *  This is used to copy a node with subnodes.
     *  @param node the node, which has to be written
     *  @return a string reader which contains the xml data
     */
    private StringReader nodesToXML (DefaultMutableTreeNode node) {
        JVPanelTreeToXML encoder = new JVPanelTreeToXML();
        nodesToXML(encoder, node);
        return new StringReader (encoder.GetString());
    }    
    
    /** Produce an XML representation of the nodetree starting from a given node
     *  @param encoder the xml encoder 
     *  @param node the node, which has to be written
     */
    private void nodesToXML (JVPanelTreeToXML encoder, DefaultMutableTreeNode node) {
        JComponent comp = (JComponent)((JVPanelTreeComponent) node.getUserObject()).getObject();
        JVPalettePanel.JVPaletteAction descr;
        if (comp instanceof JVBaseMembers)
            descr = PalettePanel.getActionForDesignModel(((JVBaseMembers)comp).getDesignModel());
        else 
            descr = PalettePanel.getActionForDesignModel(comp.getClass().getName());
        
        if (descr == null) {
            System.err.println ("Descriptor action is null for: " + comp.getClass().getName());
            return;    
        }
        encoder.writeTagStart(descr.getXMLTagName());
        encoder.writeTagAttribute("object", descr.getBaseDesignModel());
        encoder.writeTagAttribute("designmodel", descr.getDesignModel());
        encoder.writeTagEnd(false);
        encoder.componentPropertiesToXML(comp, descr.getRefComponent(), descr.getProperties());
        for (int i = 0; i < node.getChildCount(); i++)
            nodesToXML(encoder,(DefaultMutableTreeNode) node.getChildAt(i));
        encoder.writeEndTag(descr.getXMLTagName(), true);
    }
    
    /** Produce the XML representation of one component
     *  @param  comp the component to analyse
     *  @return an StringReader , which can be given to the
     *          editor panel
     */
    private StringReader compToXML(JComponent comp) {
        JVPanelTreeToXML encoder = new JVPanelTreeToXML();
        JVPalettePanel.JVPaletteAction descr;
        if (comp instanceof JVBaseMembers)
            descr = PalettePanel.getActionForDesignModel(((JVBaseMembers)comp).getDesignModel());
        else 
            descr = PalettePanel.getActionForDesignModel(comp.getClass().getName());
        
        if (descr == null) {
            System.err.println ("Descriptor action is null for: " + comp.getClass().getName());
            return null;    
        }
        encoder.writeTagStart(descr.getXMLTagName());
        encoder.writeTagAttribute("object", descr.getBaseDesignModel());
        encoder.writeTagAttribute("designmodel", descr.getDesignModel());
        encoder.writeTagEnd(false);
        encoder.componentPropertiesToXML(comp, descr.getRefComponent(), descr.getProperties());
        encoder.writeEndTag(descr.getXMLTagName(), true);
        return new StringReader (encoder.GetString());
    }
    
    /** Try to write the configuration file */
    private void writevisuFile() {
        JVPanelTreeToXML encoder = new JVPanelTreeToXML();
        nodesToXML(encoder, PanelTree.getTopNode());
        try {
            File f = new File(new URI(visuFile));
            FileWriter out = new FileWriter(f);
            out.write (encoder.GetString());
            out.close();
        } catch (Exception e) {}
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {

        JSplitPane jSplitPane1 = new JSplitPane();
        JSplitPane jSplitPane2 = new JSplitPane();
        JSplitPane jSplitPane3 = new JSplitPane();

        PanelTree = new JVPanelTree();
        PanelPropertyEditor = new JVPanelPropertyEditor();
        PalettePanel = new JVPalettePanel();
        PanelEditor = new JVPanelEditor();

        setTitle("JVisu Panel Configurator");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        jSplitPane2.setDividerLocation(200);
        jSplitPane2.setDividerSize(5);
        jSplitPane2.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane2.setResizeWeight(0.5);
        jSplitPane2.setLeftComponent(PanelTree);
        jSplitPane2.setRightComponent(PanelPropertyEditor);

        jSplitPane3.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane3.setLeftComponent(PalettePanel);
        jSplitPane3.setRightComponent(PanelEditor);

        jSplitPane1.setDividerLocation(210);
        jSplitPane1.setDividerSize(5);
        jSplitPane1.setResizeWeight(0.2);
        jSplitPane1.setLeftComponent(jSplitPane2);

        jSplitPane1.setRightComponent(jSplitPane3);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

    }
        
    /** Exit the Application 
     * @param evt the window event
     */
    private void exitForm(java.awt.event.WindowEvent evt) {
        System.exit(0);
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        SplashScreen.showSplashScreen();
        new JVPanelConfigurator().setVisible(true);
        SplashScreen.hideSplashScreen();
    }
}
