package org.jvisu.panel.configurator;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.jvisu.base.*;
import org.jvisu.panel.*;

/**
 *  <p>package:    org.jvisu.panel</p>
 *  <p>file:       JVPanelEdit.java</p>
 * $Id: JVPanelEdit.java 275 2004-04-19 08:20:55Z tgl $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>g
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 *  The purpose of JVPanelEdit is to provide a editor extension to 
 *  the JVPanel base component.
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.3
 */
public class JVPanelEdit extends JVBasePanel {

    /** update active */
    private boolean         updateactive;
    
    /** Creates a new instance of JVPanelEdit */
    public JVPanelEdit() {
        setPreferredSize(new Dimension(400, 300));
    }
    
    /** Parse a given file
     *  @param fname filename to parse
     */
    public void parseFile(String fname) {
        /* Set the basename direct to prevent
         * a propertyChangeEvent
         */
        this.base = fname;
        this.configFile = null;
        setConfigFile(fname);
    }
    
    /** Find component at position 
     *  @param pos position to look for an component
     *  @return the component at this position
     */
    public Component findComponentAt (Point pos) {
        
        Component comp = super.findComponentAt(pos);

        /* Check, if this is a JVisuBaseMember */
        while (comp != null && !(comp instanceof JVBaseMembers)) {
            comp = comp.getParent();
        }
        return comp;
    }

    /** Recursive lookup for a component
     *  @param comp component, which has to be found
     */
    public JComponent findComponent (JComponent comp) {
        return findComponent(comp, this);
    }

    /** Recursive lookup for a component
     *  @param comp component, which has to be found
     *  @param start component, to start with search
     */
    private JComponent findComponent (JComponent comp, JComponent start) {
        if (start == null || comp == null)
            return null;
        if (start == comp)
            return start;

        JComponent c;
        for (int i = 0; i < start.getComponentCount(); i++) {
            /* Hacker paranoia */
            try {
                c = (JComponent) start.getComponent(i);
            } catch (ClassCastException e) {
                continue;
            }
            JComponent subc = findComponent(comp, c);
            if (subc != null)
                return subc;
        }
        return null;
    }

    /** Get the rectangle of the component in coordinates relative to the panel
     *  origin.
     *  @param p point, which is inside the component
     *  @param r rectangle to fill with data
     *  @return rectangle or null, if no component can be found
     */
    public Rectangle getItemRectangleAt (Point p, Rectangle r) {
        return getCompRectangle(findComponentAt(p), r);
    }
        
    /** Get the rectangle of the component in coordinates relative to the panel
     *  origin.
     *  @param c the component
     *  @param r rectangle to fill with data
     *  @return rectangle or null, if no component can be found
     */
    public Rectangle getCompRectangle (Component c, Rectangle r) {
        if (r == null)
            r = new Rectangle();
        /* Invalidate, if no component given */
        if (c == null) {
            r.height = 0;
            return r;
        }    
        /* Calculate position relative to the panel origin */
        try {
            r.x = 0; r.y = 0;
            r.setSize(c.getPreferredSize());
            for (; c != this; c = c.getParent()) {
                r.x += c.getX();
                r.y += c.getY();
            }
        } catch (Exception e) {
            r.height = 0;
        }
        return r;
    }
    
    /** Read a configuration stream
     *  @param comp the component to which a element should be added
     *  @param in the inputstream to read
     */
    public void parseStream (JComponent comp, StringReader in) {
        /* Check, if the parent component is a container and can add
         * subcomponents, if not get the parent and test the parent.
         */
        while (comp != this) {
            if (!(comp instanceof JVBaseMembers) || !((JVBaseMembers) comp).isContainer()) {
                    comp = (JComponent) comp.getParent();
                    continue;
            }
            break;    
        }
        JVBasePanel.PanelHandler handler = new JVBasePanel.PanelHandler(comp);
        try {
             SAXParser p = (SAXParserFactory.newInstance()).newSAXParser();
             p.parse(new InputSource(in), handler);
        } catch (Exception e) {
             return;
        }
        revalidate();
    }
    
    /** Set added component position to the point, where the mouse click happened
     *  @param comp the added component
     *  @param p the position, where the mouse was clicked
     */
    public void setNewCompPosition(Component comp, Point p) {
        Component c;
        /* Calculate position relative to the panel origin */
        try {
            c = comp;
            do {
                c = c.getParent();
                p.x -= c.getX();
                p.y -= c.getY();
            } while (c != this);
            ((JVBaseMembers)comp).setDesignpos(p);
        } catch (Exception e) {
        }
    }
}
